import { Component, ChangeDetectorRef } from '@angular/core';
import { CommonModule } from '@angular/common';
import { ActivatedRoute, Router } from '@angular/router';
import { LootboxService } from '../services/lootbox.service';
import { LootBox, Reward } from 'app/model/LootBox';
import { NavbarComponent } from '@shared/components/navbar/navbar.component';

function shuffle<T>(array: T[]): T[] {
  const arr = array.slice();
  for (let i = arr.length - 1; i > 0; i--) {
    const j = Math.floor(Math.random() * (i + 1));
    [arr[i], arr[j]] = [arr[j], arr[i]];
  }
  return arr;
}

@Component({
  selector: 'app-lootbox-opening',
  standalone: true,
  imports: [CommonModule, NavbarComponent],
  templateUrl: './lootbox-opening.component.html',
  styleUrls: ['./lootbox-opening.component.css']
})
export default class LootboxOpeningComponent {
  lootbox: LootBox | null = null;
  isLoading = true;
  error = '';
  
  // UI State
  isOpening = false;
  isOpen = false;
  wonReward: Reward | null = null;
  prizeList: Reward[] = [];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private lootboxService: LootboxService,
    private cdr: ChangeDetectorRef
  ) {
    const idParam = this.route.snapshot.paramMap.get('id');
    if (!idParam) {
      this.error = 'Invalid lootbox ID';
      this.isLoading = false;
      return;
    }
    const lootboxId = parseInt(idParam, 10);
    this.lootboxService.getAllLootBoxes().subscribe({
      next: (lootboxes) => {
        this.lootbox = lootboxes.find(box => box.id === lootboxId) || null;
        this.isLoading = false;
        this.cdr.detectChanges();
      },
      error: () => {
        this.error = 'Failed to load lootbox data';
        this.isLoading = false;
        this.cdr.detectChanges();
      }
    });
  }

  openLootbox() {
    if (!this.lootbox || this.isOpening) return;
    this.isOpening = true;
    this.isOpen = false;
    this.wonReward = null;
    this.prizeList = []; // Clear previous prizes
    this.cdr.detectChanges();
    
    // Short delay to ensure animation plays from the beginning
    setTimeout(() => {
      this.lootboxService.purchaseLootBox(this.lootbox!.id).subscribe({
        next: (reward) => {
          this.wonReward = reward;
          this.generateCasePrizes(reward);
          this.isOpening = false;
          this.isOpen = true;
          this.cdr.detectChanges();
        },
        error: () => {
          // Fallback if API fails
          const rewards = this.lootbox!.rewards;
          const fallback = rewards[Math.floor(Math.random() * rewards.length)];
          this.wonReward = fallback;
          this.generateCasePrizes(fallback);
          this.isOpening = false;
          this.isOpen = true;
          this.cdr.detectChanges();
        }
      });
    }, 100);
  }

  generateCasePrizes(wonReward: Reward) {
    if (!this.lootbox) return;
    
    // Create a case opening display with an extremely large number of prizes for a massive scrolling animation
    const prizeCount = 200; // Set to 200 for an extremely long scrolling animation
    const winningPosition = Math.floor(prizeCount / 2); // Position of the winning prize (middle)
    
    // Get possible rewards from the lootbox
    const possibleRewards = this.lootbox.rewards;
    
    // Generate an array of random rewards
    let items: Reward[] = [];
    for (let i = 0; i < prizeCount; i++) {
      // Special handling for the winning position
      if (i === winningPosition) {
        items.push(wonReward);
      } else {
        // For all other positions, choose a random reward
        // Weight rarer items to appear less frequently
        const randomReward = this.getWeightedRandomReward(possibleRewards);
        items.push(randomReward);
      }
    }
    
    this.prizeList = items;
  }
  
  getWeightedRandomReward(rewards: Reward[]): Reward {
    // Create a weighted distribution based on probabilities
    const totalProbability = rewards.reduce((sum, reward) => sum + reward.probability, 0);
    const randomValue = Math.random() * totalProbability;
    
    let cumulativeProbability = 0;
    for (const reward of rewards) {
      cumulativeProbability += reward.probability;
      if (randomValue <= cumulativeProbability) {
        return { ...reward }; // Return a copy of the reward
      }
    }
    
    // Fallback, should never reach here
    return { ...rewards[0] };
  }

  openAgain() {
    this.isOpening = false;
    this.isOpen = false;
    this.wonReward = null;
    this.prizeList = [];
    this.cdr.detectChanges();
  }

  getBoxImage(id: number): string {
    return `/images/${id}-box.png`;
  }

  goBack(): void {
    this.router.navigate(['/game/lootboxes']);
  }

  getRarityClass(prob: number): string {
    if (prob <= 0.1) return 'text-yellow-400 border-yellow-400';
    if (prob <= 0.3) return 'text-purple-400 border-purple-400';
    return 'text-blue-400 border-blue-400';
  }

  isWonReward(reward: Reward): boolean {
    if (!this.wonReward) {
      return false;
    }
    
    return reward.id === this.wonReward.id && 
           reward.value === this.wonReward.value &&
           reward.probability === this.wonReward.probability;
  }
  
  // Calculate the center position for better alignment
  getCenterOffset(): string {
    return '0px'; // No additional offset - using animation transform instead
  }
  
  // Check if item is at center position (100th item)
  isCenterItem(index: number): boolean {
    return index === Math.floor(200 / 2); // Center item at index 100 (0-indexed)
  }
}